#!/usr/bin/env python3
# -*- coding: utf-8 -*-
################################################################################
# QuickIR : IR signal manager for IRKit
# -----------------------------------------------
# $Id: QuickIR.py 58 2016-03-12 06:09:38Z  $
#
# Copyright (C) 2015-2016 Yoshihiro Naito. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
################################################################################

import os
import sys
import re
from tkinter import *
from tkinter.messagebox import askyesno, askquestion, showerror, showinfo
from tkinter.ttk import Combobox, Treeview
from irkit import check_dir, ir_save_dir, get_irkit_info, get, post, rename, delete   # IRKit CUI
#import gettext
#gettext.install('QuickIR', 
#                os.path.join(os.path.dirname(__file__), 'locale'))

VERSION = 'QuickIR Ver.1.0.1\n'
__author__  = 'Yoshihiro Naito'
__date__    = '$Date: 2016-03-12 15:09:38 +0900 (土, 12  3 2016) $'
__version__ = '$Id: QuickIR.py 58 2016-03-12 06:09:38Z  $'
save_dir = ir_save_dir()    # Getした赤外線信号の保存先ディレクトリ名
help_win = None
#
# Colors
#
GRAY   = '#e0e0e0'
RED    = '#ff8080'
ORANGE = '#ffb040'
YELLOW = '#ffff20'
GREEN  = '#90ff90'
BLUE   = '#87c8ff'
VIOLET = '#a0a0f0'
WHITE  = '#ffffff'
TARGET = '#80a0ff'

def show_error(msg):
    '''Print error message'''
    showerror(title='QuickIR error', message=msg)

def confirm(msg):
    '''Confirm dialog'''
    return askyesno(title='QuickIR confirming', message=msg, default='no')

class Dialog(Toplevel):
    '''Dialog class'''
    def __init__(self, parent, title=None):
        Toplevel.__init__(self, parent)
        self.geometry('+{0:d}+{1:d}'.format(parent.winfo_rootx()+20,
                                            parent.winfo_rooty()+20))
        self.config(bg='white')
        self.resizable(0, 0)      # disable window resize
        self.transient(parent)    # bind to parent window

        if title:
            self.title(title)
        self.parent = parent
        self.buttonbox()
        body = Frame(self, bg=WHITE)                # make the frame
        body.grid(column=0, row=0, padx=5, pady=5)  # place the frame

        self.initial_focus = self.body(body)
        if not self.initial_focus:
            self.initial_focus = self
        self.initial_focus.focus_set()
        self.protocol('WM_DELETE_WINDOW', self.cancel)

        self.grab_set()
        self.wait_window(self)

    def body(self, master):
        pass

    def buttonbox(self):
        box = Frame(self, bg='white')
        self.button1 = b1 = Button(box, text='OK', width=10, bg=WHITE, command=self.ok, default=ACTIVE)
        self.button2 = b2 = Button(box, text='Cancel', width=10, bg=WHITE, command=self.cancel)
        b1.pack(side=LEFT, padx=5, pady=5)
        b2.pack(side=LEFT, padx=5, pady=5)
        self.bind('<Return>', self.ok)
        self.bind('<Escape>', self.cancel)
        box.grid(column=0, row=1)

    def ok(self, event=None):
        if not self.validate():
            self.initial_focus.focus_set()
            return
        self.withdraw()
        self.update_idletasks()
        self.apply()
        self.cancel()

    def cancel(self, event=None):
        self.parent.focus_set()
        self.destroy()

    def validate(self):
        return True

    def apply(self):
        pass

class NewFolder_Dialog(Dialog):
    '''Create new folder dialog'''
    def __init__(self, parent, title, init_val=None):
        self.init_val = init_val
        self.new_name = None
        super().__init__(parent, title)

    def body(self, master):
        '''Dialog body'''
        Label(master, text='New Folder:', bg=WHITE).grid(column=0, row=0)
        self.en = en = Entry(master, width=20)
        en.grid(column=1, row=0)
        if self.init_val:
            en.insert(END, self.init_val)
        en.focus()

    def apply(self):
        '''Callback function of OK button'''
        self.new_name = self.en.get()

def ask_new_foldername(master, title, init_val=None):
    '''Ask new folder name Dialog'''
    return NewFolder_Dialog(master, title, init_val).new_name

def directories():
    '''List of only directories'''
    path = save_dir + '/signals'
    return ['/'] + [d for d in os.listdir(path) if d[0] != '.' and os.path.isdir(path + '/' + d)]

def dir_list():
    '''Return list of directories and files'''
    return [l for l in os.walk(save_dir + '/signals')]

def create_directory(dirname):
    '''Create directory'''
    os.mkdir(save_dir + '/signals/' + dirname)

def rename_directory(old, new):
    '''Rename directory'''
    os.rename(save_dir + '/signals/' + old, save_dir + '/signals/' + new)

def remove_directory(dirname):
    '''Remove directory with included files'''
    for file in os.listdir(save_dir + '/signals/' + dirname):
        os.remove(save_dir + '/signals/' + dirname + '/' + file)
    os.rmdir(save_dir + '/signals/' + dirname)

def move_file(src, dst):
    '''Move .json file'''
    os.rename(save_dir + '/signals/' + src + '.json',
              save_dir + '/signals/' + dst + '.json')


class DragDrop:
    '''Drag and Drop class for Treeview'''
    treeview = None
    dragging_item = dragging_did = dir_bg = None

    def get_treeview(self):
        return DragDrop.treeview

    def make_binds(self):
        DragDrop.treeview.tag_bind(self.id, '<Button-1>', self.drag_start)      # enable dragging
        DragDrop.treeview.tag_bind(self.id, '<Button1-Motion>', self.dragging)  # enable dragging
        DragDrop.treeview.tag_bind(self.id, '<ButtonRelease>', self.drop)       # enable dragging

    def change_mouse_cursor(self, cursor):
        self.get_treeview().config(cursor=cursor)

    def set_drag_item(self, item):
        DragDrop.dragging_item = item

    def get_dragging_item(self):
        return DragDrop.dragging_item

    def save_bg(self, did, color):
        '''save background color'''
        DragDrop.dragging_did = did
        DragDrop.dir_bg = color

    def restore_bg(self):
        '''restore saved background color'''
        if DragDrop.dragging_did:
            DragDrop.treeview.tag_configure(DragDrop.dragging_did, background=DragDrop.dir_bg)
            self.save_bg(None, None)

    def drag_start(self, event):
        pass

    def dragging(self, event):
        if self.get_dragging_item():
            self.change_mouse_cursor('icon')
        self.restore_bg()

    def drop(self, event):
        self.restore_bg()
        self.set_drag_item(None)
        self.change_mouse_cursor('')

class NewTreeView(DragDrop):
    def __init__(self, master, column, height, selectmode):
        DragDrop.treeview = tv = Treeview(master, column=column, height=height, selectmode=selectmode)
        tv.bind('<ButtonRelease>', self.drop)

class NewTreeviewFile(DragDrop):
    def __init__(self, dir, file, position='end'):
        treeview = self.get_treeview()
        self.id = fid = treeview.insert(dir, position, values=(re.escape(os.path.splitext(file)[0])))
        treeview.item(fid, tags=fid)
        # ディレクトリのカラーラベルを要素(ファイル)にも設定
#       color = treeview.tag_configure(dir, option='background')
#       treeview.tag_configure(fid, background=color)
        self.make_binds()      # setup binds

    def drag_start(self, event):
        '''start file dragging'''
        self.set_drag_item(item=self.id)   # set the drag item
        self.save_bg(did=None, color=None) # initialize the drop target

class NewTreeviewDir(DragDrop):
    def __init__(self, level, dir):
        treeview = self.get_treeview()
        if dir == '/':
            self.id = did = treeview.insert(level, 'end', iid=dir, text=dir, tags=dir, open=True)
        else:
            self.id = did = treeview.insert(level, 'end', text=dir, open=True)
            treeview.item(did, tags=did)
        self.make_binds()      # setup binds

    def dragging(self, event):
        '''dragging to a directory'''
        if not self.get_dragging_item():
            return
        treeview = self.get_treeview()
        if not DragDrop.dragging_did:
            self.save_bg(self.id, treeview.tag_configure(self.id, option='background'))   # 元のbackgroundを退避
            treeview.tag_configure(self.id, background=TARGET)
        elif DragDrop.dragging_did != self.id:
            self.restore_bg()
            self.save_bg(self.id, treeview.tag_configure(self.id, option='background'))   # 元のbackgroundを退避
            treeview.tag_configure(self.id, background=TARGET)

    def drop(self, event):
        '''drop the file to the directory'''
        fid = self.get_dragging_item()
        if not fid:
            return
        treeview = self.get_treeview()
        fnm = treeview.item(fid)['values'][0]
        did = treeview.parent(fid)

        if did == self.id:              # 同じディレクトリ内へのドロップ
            DragDrop.drop(self, event)
            return

        # 移動先ディレクトリに同名の要素(ディレクトリ、ファイル)が無いかチェックする
        for id in treeview.get_children(self.id):
            if not treeview.item(id)['values']:
                if treeview.item(id)['text'] == fnm:
                    showerror(title='error', message='移動先に同名のフォルダがあるため、移動できません。')
                    return
            elif treeview.item(id)['values'][0] == fnm:
                if askquestion('Move file', '''\
移動先のリモコンフォルダに同名の赤外線信号ファイルが存在しています。
移動すると、移動先の赤外線信号ファイルは移動元のファイルで置き換えられます。
移動してもよろしいですか？''', icon='warning', default='no') == 'yes':
                    treeview.delete(id)
                else:
                    DragDrop.drop(self, event)
                    return
                break
        #
        try:
            move_file(src=treeview.item(did)['text'] + '/' + fnm,
                      dst=treeview.item(self.id)['text'] + '/' + fnm)
        except:
            showerror(title='error', message='ファイルの移動に失敗しました。')
            return
        treeview.move(fid, self.id, 0)    # ディレクトリの先頭に移動する
        DragDrop.drop(self, event)
        # ドロップした要素のカラーラベル変更
#       tag = treeview.item(fid, option='tags')[0]
#       color = treeview.tag_configure(self.id, option='background')
#       treeview.tag_configure(tag, background=color)


class Application(Frame):
    '''QuickIR application class'''

    def __init__(self, master):
        self.master = master
        master.title('QuickIR')     # Title of the Application windows
        master.minsize(268, 200)    # ウィンドウの最小サイズを制限
        Frame.__init__(self, master, borderwidth=4, bg=GRAY)
        self.pack(expand=True, fill='both')
        self.createMenu(master)         # create menu
        self.createContextMenu(master)  # create context menu
        self.createWidgets()            # アプリケーションウィンドウのウィジェット作成
        self.makeBind(master)           # ショートカット設定
        self.context_menu_event = None

    def get_signal(self, event=None):
        '''Get IR signal Dialog'''

        def treeview_dir2did(treeview, dir):
            '''Return the treeview identifier which corresponds to the name dir'''
            for did in treeview.get_children() + treeview.get_children('/'):
                if treeview.item(did)['text'] == dir:
                    return did
            else:
                return None

        class QIR_GetDialog(Dialog):
            '''Get Dialog of QuickIR'''
            def __init__(self, master, treeview, title, description):
                self.description = description
                self.treeview = treeview
                self.saved = False
                super().__init__(master, title)

            def body(self, master):
                # Image
                self.image = PhotoImage(file='images/IRKit.gif')
                Label(master, image=self.image, bg=WHITE).grid(column=0, row=1, rowspan=3)
                # Description
                Label(master, text=self.description, bg=WHITE).grid(column=0, row=0, columnspan=4)
                # Button label
                self.button1.config(text='Save')
                # Entry
                Label(master, text='フォルダ:', bg=WHITE).grid(column=1, row=1, sticky='ws')
                dlist = directories()
                self.cb = cb = Combobox(master, width=22,
                                        height=len(dlist)+1, justify='left',
                                        state='readonly', values=dlist)
                cb.set('/')
                cb.grid(column=2, row=1, columnspan=2, sticky='ws')
                Label(master, text='信号名:', bg=WHITE).grid(column=1, row=2, sticky='ws')
                self.en = en = Entry(master, width=24)
                en.grid(column=2, row=2, padx=8, columnspan=2, sticky='ws')
                en.focus()
                return self.en

            def apply(self):
                self.dir = dir = self.cb.get()
                self.sig = signame = self.en.get()
                if signame:
                    # same file already exists ?
                    treeview = self.treeview
                    did = treeview_dir2did(treeview, dir)
                    for fid in treeview.get_children(did):
                        if treeview.item(fid)['values'][0] == signame:
                            show_error('保存先のリモコンフォルダには\"{0}\"と同名の赤外線信号ファイルがあるため、保存できません。'.format(signame))
                            return
                    info = get_irkit_info()
                    if get(info['IPaddress'], info['Port'], info['Host'], dir + '/' + signame):  # irkit.get()
                        self.saved = True
                    else:
                        show_error('受信した赤外線信号がありません。')
                else:
                    show_error('赤外線信号名を入力してください。')

        self.context_menu_event = None       # clear context menu event
        d = QIR_GetDialog(self.master, self.tv, 
                          title='Get IR signal',
                          description='最後に受信した赤外線信号を保存します。')
        if d.saved:
            did = treeview_dir2did(self.tv, d.dir)
            fid = NewTreeviewFile(did, d.sig, position=0).id
            self.tv.tag_bind(fid, '<Return>',   self.send_signal)  # リターンキーで送信
            self.tv.tag_bind(fid, '<Double-1>', self.send_signal)  # 左ダブルクリックで送信
        self.focus()

    def get_selected_item(self, event=None):
        '''get the selected TreeView's item'''
        if self.context_menu_event:
            event = self.context_menu_event
            treeview = event.widget
            self.context_menu_event = None       # clear context menu event
            fid = treeview.identify('item', event.x, event.y)   # clicked file identifier
        else:
            treeview = self.tv
            fid = treeview.selection()
        if treeview.item(fid)['values']:
            file = treeview.item(fid)['values'][0]
            did = treeview.parent(fid)
            dir = treeview.item(did)['text']
            return dir, fid, file
        else:
            return treeview.item(fid)['text'], None, None

    def send_signal(self, event=None):
        '''send the IR-signal'''
        dir, fid, signame = self.get_selected_item(event)
        if signame:
            info = get_irkit_info()
            try:
                post(info['IPaddress'], info['Port'], dir + '/' + signame)  # irkit.post()
            except:
                show_error('\"{0}\"赤外線信号の送信が失敗しました。'.format(signame))
            else:
                showinfo(title='送信成功', message='\"{0}\"を送信しました。'.format(dir + '/' + signame))
        else:
            show_error('赤外線信号が選択されていません。')
        self.focus()

    def rename_signal(self, event=None):
        '''rename the IR-signal'''

        class QIR_RenDialog(Dialog):
            '''Rename Dialog of QuickIR'''
            def __init__(self, master, title, description):
                self.description = description
                self.renamed = False
                self.new_name = None
                super().__init__(master, title)

            def body(self, master):
                # Description
                Label(master, text=self.description, bg=WHITE).grid(column=0, row=0, columnspan=4)
                # Button label
                self.button1.config(text='Rename')
                # Entry
                Label(master, text='新しい信号名:', bg=WHITE).grid(column=1, row=1, sticky='ws')
                self.en = en = Entry(master, width=24)
                en.insert(END, old_name)
                en.grid(column=2, row=1, padx=8, columnspan=2, sticky='s')
                en.focus()
                return self.en

            def apply(self):
                '''Rename IR signal filename'''
                new_name = self.en.get()
                if new_name:
                    try:
                        rename(dir + '/' + old_name, dir + '/' + new_name)
                    except:
                        show_error('赤外線信号名の変更に失敗しました。')
                        return
                    self.renamed = True
                    self.new_name = new_name
                else:
                    show_error('赤外線信号名を入力してください。')

        dir, fid, old_name = self.get_selected_item(event)
        if not old_name:
            show_error('赤外線信号が選択されていません。')
        else:
            msg = '\"{0}/{1}\"の名前を変更します。'.format(dir, old_name)
            d = QIR_RenDialog(self.master, title='Rename IR signal', description=msg)
            if d.renamed:
                self.tv.item(fid, values=(re.escape(d.new_name)))
        self.focus()

    def delete_signal(self, event=None):
        '''赤外線信号の削除'''
        dir, fid, signame = self.get_selected_item(event)
        if signame:
            if confirm('\"{0}/{1}\"を削除します。よろしいですか？'.format(dir, signame)):
                try:
                    delete(dir + '/' + signame)      # irkit.delete()
                except:
                    show_error('赤外線信号の削除に失敗しました。')
                else:
                    self.tv.delete(fid)
        else:
            show_error('赤外線信号が選択されていません。')
        self.focus()

    def createMenu(self, master):
        '''メニューの作成'''
        # create menu bar
        menubar = Menu(master)
        master.config(menu=menubar)
        # File menu
        self.file_menu = file_menu = Menu(menubar, tearoff=False)
        menubar.add_cascade(label='ファイル(F)', under=5, menu=file_menu)
        file_menu.add_command(label='リモコンフォルダ作成(N)...', under=11, command=self.create_new_folder,
                              accelerator='Ctrl-Command-N' if sys.platform == 'darwin' else 'Shift+Ctrl+N')
        file_menu.add_command(label='リモコンフォルダ名変更(R)...', under=12, command=self.rename_folder,
                              accelerator='Ctrl-Command-R' if sys.platform == 'darwin' else 'Shift+Ctrl+R')
        file_menu.add_command(label='リモコンフォルダ削除...', command=self.delete_folder)
        file_menu.add_separator()
        file_menu.add_command(label='QuickIR終了', under=0, command=sys.exit,
                              accelerator=None if sys.platform == 'darwin' else 'Ctrl+Q')

        # IRKit menu
        self.irkit_menu = irkit_menu = Menu(menubar, tearoff=False)
        menubar.add_cascade(label='QuickIR', underline=0, menu=irkit_menu)
        # IRKit menu items
        irkit_menu.add_command(label='IR信号保存(S)...',   under=7, command=self.get_signal,
                               accelerator='Command-S' if sys.platform == 'darwin' else 'Ctrl+S')
        irkit_menu.add_command(label='IR信号送信(P)',      under=7, command=self.send_signal,
                               accelerator='Command-P' if sys.platform == 'darwin' else 'Ctrl+P')
        irkit_menu.add_command(label='IR信号名変更(R)...', under=8, command=self.rename_signal,
                               accelerator='Command-R' if sys.platform == 'darwin' else 'Ctrl+R')
        irkit_menu.add_command(label='IR信号削除',                  command=self.delete_signal)

        # HELP menu
        self.help_menu = help_menu = Menu(menubar, tearoff=False)
        menubar.add_cascade(label='ヘルプ(H)', underline=4, menu=help_menu)
        # HELP menu items
        help_menu.add_command(label='ヘルプ(H)...', under=4, command=self.help_window)
        help_menu.add_command(label='QuickIRについて(A)...', under=12, command=self.about_QuickIR)

    def createContextMenu(self, master):
        '''コンテキストメニューの作成'''
        self.ctxt_menu_folder = menu_folder = Menu(master, tearoff=False)
        for l, com in (('リモコンフォルダ作成...',   self.create_new_folder),
                       ('リモコンフォルダ名変更...', self.rename_folder),
                       ('リモコンフォルダ削除',      self.delete_folder)):
            menu_folder.add_command(label=l, command=com)

        self.ctxt_menu_file = menu_file = Menu(master, tearoff=False)
        for l, com in (('IR信号保存...',    self.get_signal),
                       ('IR信号送信',       self.send_signal),
                       ('IR信号名変更...',  self.rename_signal),
                       ('IR信号削除',       self.delete_signal)):
            menu_file.add_command(label=l, command=com)

    def popup_context_menu(self, event=None):
        '''popup context menu'''
        self.context_menu_event = event
        widget = event.widget
        id = widget.identify('item', event.x, event.y)   # 対象の要素識別子
        widget.selection_set(id)
        #
        column = widget.identify_column(event.x)
        if column == '#0':
            # right clicked on the tree (1st) column
            self.ctxt_menu_folder.post(event.x_root, event.y_root)
        elif column == '#1':
            # right clicked on the item (2nd) column
            self.ctxt_menu_file.post(event.x_root, event.y_root)

    def create_new_folder(self, event=None):
        '''Create new folder'''
        self.context_menu_event = None       # clear context menu event
        folder_name = ask_new_foldername(self.master, title='新規フォルダ名の入力')
        if folder_name:
            if folder_name[0] in ('.', '/', '\\'):
                showerror(title='error', message='このフォルダ名は使用できません。')
            else:
                try:
                    create_directory(folder_name)
                except:
                    showerror(title='error', message='フォルダを作成できませんでした。')
                else:
                    NewTreeviewDir(level='/', dir=folder_name)
        else:
            showerror(title='error', message='フォルダ名が入力されていません。')
        self.focus()

    def rename_folder(self, event=None):
        '''Rename folder name'''
        if self.context_menu_event:
            event = self.context_menu_event
            self.context_menu_event = None       # clear context menu event
            treeview = event.widget
            did = treeview.identify('item', event.x, event.y)   # clicked directory identifier
        else:
            treeview = self.tv
            did = treeview.selection()
        dir = treeview.item(did)['text']
        if not dir:
            showerror(title='error', message='リモコンフォルダが選択されていません。')
            self.focus()
            return
        elif dir == '/':
            showerror(title='error', message='このフォルダ名は変更できません。')
            self.focus()
            return
        #
        new_name = ask_new_foldername(self.master, title='フォルダ名の入力', init_val=dir)
        if not new_name: return
        if new_name[0] in ('.', '/', '\\'):
            showerror(title='error', message='このフォルダ名には変更できません。')
        else:
            try:
                rename_directory(dir, new_name)
            except:
                showerror(title='error', message='リモコンフォルダ名の変更に失敗しました。')
            else:
                treeview.item(did, text=new_name)
        self.focus()

    def delete_folder(self, event=None):
        '''Delete folder'''
        if self.context_menu_event:
            event = self.context_menu_event
            self.context_menu_event = None       # clear context menu event
            treeview = event.widget
            did = treeview.identify('item', event.x, event.y)   # clicked directory identifier
        else:
            treeview = self.tv
            did = treeview.selection()
        dir = treeview.item(did)['text']
        if not dir:
            showerror(title='error', message='リモコンフォルダが選択されていません。')
        elif dir == '/':
            showerror(title='error', message='このフォルダは削除できません。')
        elif askquestion('Delete folder', '''\
リモコンフォルダを削除すると、その配下のIR信号ファイルも削除されます。
\"{0}\"フォルダを削除してもよろしいですか？'''.format(dir), icon='warning', default='no') == 'yes':
            try:
                remove_directory(dir)
            except:
                showerror(title='error', message='このフォルダは削除できません。')
            else:
                treeview.delete(did)
        self.focus()

    def create_tree_sub(self, tree, level, lists, index):
        '''create directories tree sub'''
        path, dirs, files = lists[index]
        if files:
            for file in sorted(files):
                if file[0] == '.':    # 先頭が'.'のファイルは除く
                    continue
                fid = NewTreeviewFile(level, file).id
                self.tv.tag_bind(fid, '<Return>',   self.send_signal)  # リターンキーで送信
                self.tv.tag_bind(fid, '<Double-1>', self.send_signal)  # 左ダブルクリックで送信
        if dirs:
            for i, dir in enumerate(dirs):
                if dir[0] == '.':     # 先頭が'.'のディレクトリは除く
                    index += 1
                    continue
                did = NewTreeviewDir(level, dir).id
                index += self.create_tree_sub(tree, did, lists[index+1:], 0)
        return index+1

    def create_tree(self):
        '''create directories tree'''
        self.tv.delete(*self.tv.get_children())             # Clear all items
        NewTreeviewDir(level='', dir='/')                   # ルートディレクトリ(特殊フォルダ)
        self.create_tree_sub(tree=self.tv, level='/', lists=dir_list(), index=0)

    def createWidgets(self):
        '''put widgets on the application main window'''
        toolbar = Frame(self, borderwidth=0, background=WHITE, height=32)
        self.createToolbar(toolbar)
        # TreeView
        self.tv = tv = NewTreeView(self, column=2, height=24, selectmode='browse').get_treeview()
        tv['columns']=('Signal')
        tv.column('#0', width=120)
        tv.column('Signal', width=160)
        tv.heading('#0', text='リモコン')
        tv.heading('Signal', text='IR信号名')
        self.create_tree()
        # Scrollbar
        sb_y = Scrollbar(self, orient=VERTICAL, command=tv.yview)
        tv.configure(yscrollcommand=sb_y.set)
        # Listbox, ScrollbarをGridderで配置
        toolbar.grid(column=0, columnspan=2, row=0, sticky='w')
        tv.grid(column=0, row=1, sticky='news')
        sb_y.grid(column=1, row=1, sticky='ns')
        # リサイズ設定
        self.grid_columnconfigure(index=0, weight=1)
        self.grid_rowconfigure(index=1, weight=1)
        # 右クリックでコンテキストメニュー
        tv.bind('<Button-2>' if sys.platform == 'darwin' else '<Button-3>', self.popup_context_menu)

    def createToolbar(self, toolbar):
        '''make toolbar'''
        self.img_folder_add = PhotoImage(file='images/folder_add.gif')
        self.img_folder_ren = PhotoImage(file='images/folder_rename.gif')
        self.img_folder_del = PhotoImage(file='images/folder_delete.gif')
        self.img_file_add   = PhotoImage(file='images/file_add.gif')
        self.img_file_post  = PhotoImage(file='images/file_post.gif')
        self.img_file_ren   = PhotoImage(file='images/file_rename.gif')
        self.img_file_del   = PhotoImage(file='images/file_delete.gif')
        for col, (img, com) in enumerate([(self.img_folder_add, self.create_new_folder),
                                          (self.img_folder_ren, self.rename_folder),
                                          (self.img_folder_del, self.delete_folder),
                                          (self.img_file_add,   self.get_signal),
                                          (self.img_file_post,  self.send_signal),
                                          (self.img_file_ren,   self.rename_signal),
                                          (self.img_file_del,   self.delete_signal)]):
            Button(toolbar, image=img, bg=WHITE, relief=FLAT, command=com).grid(column=col, row=0, sticky='w')

    def makeBind(self, master):
        '''setup keyboard short-cut'''
        for event, com in [('<Control-Command-KeyPress-N>' if sys.platform == 'darwin' else '<Shift-Control-KeyPress-N>', self.create_new_folder),
                           ('<Control-Command-KeyPress-R>' if sys.platform == 'darwin' else '<Shift-Control-KeyPress-R>', self.rename_folder),
                           ('<Command-KeyPress-s>' if sys.platform == 'darwin' else '<Control-KeyPress-s>', self.get_signal),
                           ('<Command-KeyPress-p>' if sys.platform == 'darwin' else '<Control-KeyPress-p>', self.send_signal),
                           ('<Command-KeyPress-r>' if sys.platform == 'darwin' else '<Control-KeyPress-r>', self.rename_signal),
                           ('<Command-KeyPress-q>' if sys.platform == 'darwin' else '<Control-KeyPress-q>', sys.exit)]:
            master.bind(event, com)

    def help_window(self):
        '''Help window'''
        global help_win
        if help_win == None or not help_win.winfo_exists():
            help_win = Toplevel()           # ヘルプ用の新しいウィンドウを作る
            help_win.transient(self.master) # 親ウィンドウに結び付ける
            help_win.title('QuickIR Help')  # ウィンドウのタイトル
            help_win.configure(bg=WHITE)    # 背景色を白にする
            help_win.minsize(400, 200)
            help_win.resizable(0, 0)        # disable window resize

            Message(help_win, text='QuickIRの使用方法', bg=WHITE, width=400, justify='left').pack(anchor='c', side='top')
            Message(help_win, text='ツールバー:', bg=WHITE, width=400, justify='left').pack(anchor='w', side='top')
            f = Frame(help_win, bg=WHITE)
            Label(f, image=self.img_folder_add,         bg=WHITE).grid(column=0, row=1, sticky='w')
            Label(f, text=' リモコンフォルダ作成(N)',   bg=WHITE).grid(column=1, row=1, sticky='w')
            Label(f, image=self.img_folder_ren,         bg=WHITE).grid(column=0, row=2, sticky='w')
            Label(f, text=' リモコンフォルダ名変更(R)', bg=WHITE).grid(column=1, row=2, sticky='w')
            Label(f, image=self.img_folder_del,         bg=WHITE).grid(column=0, row=3, sticky='w')
            Label(f, text='　リモコンフォルダ削除',     bg=WHITE).grid(column=1, row=3, sticky='w')
            Label(f, image=self.img_file_add,           bg=WHITE).grid(column=0, row=4, sticky='w')
            Label(f, text='　IR信号保存(S)',            bg=WHITE).grid(column=1, row=4, sticky='w')
            Label(f, image=self.img_file_post,          bg=WHITE).grid(column=0, row=5, sticky='w')
            Label(f, text='　IR信号送信(P)',            bg=WHITE).grid(column=1, row=5, sticky='w')
            Label(f, image=self.img_file_ren,           bg=WHITE).grid(column=0, row=6, sticky='w')
            Label(f, text='　IR信号名変更(R)',          bg=WHITE).grid(column=1, row=6, sticky='w')
            Label(f, image=self.img_file_del,           bg=WHITE).grid(column=0, row=7, sticky='w')
            Label(f, text='　IR信号削除',               bg=WHITE).grid(column=1, row=7, sticky='w')
            f.pack(anchor='w', side='top')

            help_text = '''
ファイルメニュー:
  [リモコンフォルダ作成(N)...]\tリモコンのフォルダ作成
  [リモコンフォルダ名変更(R)...]\tリモコンのフォルダ名変更
  [リモコンフォルダ削除...]\tリモコンのフォルダ削除
  [QuickIR終了]\t\tQuickIR終了\n
QuickIRメニュー:
  [IR信号保存(S)...]\t受信した赤外線信号を保存
  [IR信号送信(P)]\t選択した赤外線信号を送信
  [IR信号名変更(R)...]\t赤外線信号の名前を変更
  [IR信号削除]\t選択した赤外線信号を削除\n
ヘルプメニュー:
  [ヘルプ(H)...]\t\tヘルプの表示
  [QuickIRについて(A)...]\tQuickIRについて'''
            Message(help_win, text=help_text, bg=WHITE, width=400, justify='left').pack(anchor='w', side='top')

    def about_QuickIR(self):
        '''About message'''
        showinfo(title='about', message=VERSION + __version__ + '\nCopyright(C) N@i 2015-2016')
        self.focus()


def main():
    check_dir(save_dir)
    check_dir(save_dir + '/signals')

    root = Tk()
    app = Application(master=root)
    app.mainloop()

if __name__ == '__main__':
    main()
